/*
 * Copyright 2022-2023 Sensaru GmbH
 */

#ifndef LIBC1_MODULE_SRC_ACL_USERCOMMON_H
#define LIBC1_MODULE_SRC_ACL_USERCOMMON_H

#include <homegear-base/Variable.h>

namespace C1Module {

enum class UserType {
  unknown,
  person,
  authService,
};

enum class PrincipalLevel {
  unknown = 0,
  superuser = 1,
  systemProvider = 2,
  systemDistributor = 3,
  businessPartner = 4,
  enduser = 5,
  homeClient = 6,
  module = 7
};

struct PrincipalId {
  static PrincipalId FromStruct(const BaseLib::PVariable &value) {
    PrincipalId principalId;

    auto valueIterator = value->structValue->find("sp");
    if (valueIterator == value->structValue->end()) return {};
    principalId.sp = valueIterator->second->stringValue;

    valueIterator = value->structValue->find("sd");
    if (valueIterator == value->structValue->end()) return {};
    principalId.sd = valueIterator->second->stringValue;

    valueIterator = value->structValue->find("bp");
    if (valueIterator == value->structValue->end()) return {};
    principalId.bp = valueIterator->second->stringValue;

    return principalId;
  }

  [[nodiscard]] BaseLib::PVariable ToStruct() const {
    auto principalStruct = std::make_shared<BaseLib::Variable>(BaseLib::VariableType::tStruct);
    principalStruct->structValue->emplace("sp", std::make_shared<BaseLib::Variable>(sp));
    principalStruct->structValue->emplace("sd", std::make_shared<BaseLib::Variable>(sd));
    principalStruct->structValue->emplace("bp", std::make_shared<BaseLib::Variable>(bp));
    return principalStruct;
  }

  std::string sp;
  std::string sd;
  std::string bp;

  bool operator!=(const PrincipalId &rhs) const {
    return !(operator==(rhs));
  }

  bool operator==(const PrincipalId &rhs) const {
    return sp == rhs.sp && sd == rhs.sd && bp == rhs.bp;
  }
};

struct FullUserId {
  static FullUserId fromStruct(const BaseLib::PVariable &value) {
    FullUserId userId;

    auto valueIterator = value->structValue->find("type");
    if (valueIterator == value->structValue->end()) return {};
    userId.type = (PrincipalLevel)valueIterator->second->integerValue;

    valueIterator = value->structValue->find("sp");
    if (valueIterator == value->structValue->end()) return {};
    userId.principal.sp = valueIterator->second->stringValue;

    valueIterator = value->structValue->find("sd");
    if (valueIterator == value->structValue->end()) return {};
    userId.principal.sd = valueIterator->second->stringValue;

    valueIterator = value->structValue->find("bp");
    if (valueIterator == value->structValue->end()) return {};
    userId.principal.bp = valueIterator->second->stringValue;

    valueIterator = value->structValue->find("id");
    if (valueIterator == value->structValue->end()) return {};
    userId.user = valueIterator->second->stringValue;

    return userId;
  }

  [[nodiscard]] BaseLib::PVariable toStruct() const {
    auto userStruct = std::make_shared<BaseLib::Variable>(BaseLib::VariableType::tStruct);
    userStruct->structValue->emplace("type", std::make_shared<BaseLib::Variable>((int32_t)type));
    userStruct->structValue->emplace("sp", std::make_shared<BaseLib::Variable>(principal.sp));
    userStruct->structValue->emplace("sd", std::make_shared<BaseLib::Variable>(principal.sd));
    userStruct->structValue->emplace("bp", std::make_shared<BaseLib::Variable>(principal.bp));
    userStruct->structValue->emplace("id", std::make_shared<BaseLib::Variable>(user));
    return userStruct;
  }

  PrincipalLevel type = PrincipalLevel::unknown;
  PrincipalId principal;
  std::string user;

  bool operator!=(const FullUserId &rhs) const {
    return !(operator==(rhs));
  }

  bool operator==(const FullUserId &rhs) const {
    return user == rhs.user && principal == rhs.principal;
  }
};

}

#endif //LIBC1_MODULE_SRC_ACL_USERCOMMON_H
